#!/usr/bin/env python

'''
MOSSE tracking sample

This sample implements correlation-based tracking approach, described in [1].

Usage:
  mosse.py [--pause] [<video source>]

  --pause  -  Start with playback paused at the first video frame.
              Useful for tracking target selection.

  Draw rectangles around objects with a mouse to track them.

Keys:
  SPACE    - pause video
  c        - clear targets

[1] David S. Bolme et al. "Visual Object Tracking using Adaptive Correlation Filters"
    http://www.cs.colostate.edu/~draper/papers/bolme_cvpr10.pdf
'''

# Python 2/3 compatibility
from __future__ import print_function
import sys
PY3 = sys.version_info[0] == 3

if PY3:
    xrange = range

import numpy as np
import cv2 as cv
from common import draw_str, RectSelector
import video

def rnd_warp(a):
    h, w = a.shape[:2]
    T = np.zeros((2, 3))
    coef = 0.2
    ang = (np.random.rand()-0.5)*coef
    c, s = np.cos(ang), np.sin(ang)
    T[:2, :2] = [[c,-s], [s, c]]
    T[:2, :2] += (np.random.rand(2, 2) - 0.5)*coef
    c = (w/2, h/2)
    T[:,2] = c - np.dot(T[:2, :2], c)
    return cv.warpAffine(a, T, (w, h), borderMode = cv.BORDER_REFLECT)

def divSpec(A, B):
    Ar, Ai = A[...,0], A[...,1]
    Br, Bi = B[...,0], B[...,1]
    C = (Ar+1j*Ai)/(Br+1j*Bi)
    C = np.dstack([np.real(C), np.imag(C)]).copy()
    return C

eps = 1e-5

class MOSSE:
    def __init__(self, frame, rect):
        x1, y1, x2, y2 = rect
        w, h = map(cv.getOptimalDFTSize, [x2-x1, y2-y1])
        x1, y1 = (x1+x2-w)//2, (y1+y2-h)//2
        self.pos = x, y = x1+0.5*(w-1), y1+0.5*(h-1)
        self.size = w, h
        img = cv.getRectSubPix(frame, (w, h), (x, y))

        self.win = cv.createHanningWindow((w, h), cv.CV_32F)
        g = np.zeros((h, w), np.float32)
        g[h//2, w//2] = 1
        g = cv.GaussianBlur(g, (-1, -1), 2.0)
        g /= g.max()

        self.G = cv.dft(g, flags=cv.DFT_COMPLEX_OUTPUT)
        self.H1 = np.zeros_like(self.G)
        self.H2 = np.zeros_like(self.G)
        for _i in xrange(128):
            a = self.preprocess(rnd_warp(img))
            A = cv.dft(a, flags=cv.DFT_COMPLEX_OUTPUT)
            self.H1 += cv.mulSpectrums(self.G, A, 0, conjB=True)
            self.H2 += cv.mulSpectrums(     A, A, 0, conjB=True)
        self.update_kernel()
        self.update(frame)

    def update(self, frame, rate = 0.125):
        (x, y), (w, h) = self.pos, self.size
        self.last_img = img = cv.getRectSubPix(frame, (w, h), (x, y))
        img = self.preprocess(img)
        self.last_resp, (dx, dy), self.psr = self.correlate(img)
        self.good = self.psr > 8.0
        if not self.good:
            return

        self.pos = x+dx, y+dy
        self.last_img = img = cv.getRectSubPix(frame, (w, h), self.pos)
        img = self.preprocess(img)

        A = cv.dft(img, flags=cv.DFT_COMPLEX_OUTPUT)
        H1 = cv.mulSpectrums(self.G, A, 0, conjB=True)
        H2 = cv.mulSpectrums(     A, A, 0, conjB=True)
        self.H1 = self.H1 * (1.0-rate) + H1 * rate
        self.H2 = self.H2 * (1.0-rate) + H2 * rate
        self.update_kernel()

    @property
    def state_vis(self):
        f = cv.idft(self.H, flags=cv.DFT_SCALE | cv.DFT_REAL_OUTPUT )
        h, w = f.shape
        f = np.roll(f, -h//2, 0)
        f = np.roll(f, -w//2, 1)
        kernel = np.uint8( (f-f.min()) / np.ptp(f)*255 )
        resp = self.last_resp
        resp = np.uint8(np.clip(resp/resp.max(), 0, 1)*255)
        vis = np.hstack([self.last_img, kernel, resp])
        return vis

    def draw_state(self, vis):
        (x, y), (w, h) = self.pos, self.size
        x1, y1, x2, y2 = int(x-0.5*w), int(y-0.5*h), int(x+0.5*w), int(y+0.5*h)
        cv.rectangle(vis, (x1, y1), (x2, y2), (0, 0, 255))
        if self.good:
            cv.circle(vis, (int(x), int(y)), 2, (0, 0, 255), -1)
        else:
            cv.line(vis, (x1, y1), (x2, y2), (0, 0, 255))
            cv.line(vis, (x2, y1), (x1, y2), (0, 0, 255))
        draw_str(vis, (x1, y2+16), 'PSR: %.2f' % self.psr)

    def preprocess(self, img):
        img = np.log(np.float32(img)+1.0)
        img = (img-img.mean()) / (img.std()+eps)
        return img*self.win

    def correlate(self, img):
        C = cv.mulSpectrums(cv.dft(img, flags=cv.DFT_COMPLEX_OUTPUT), self.H, 0, conjB=True)
        resp = cv.idft(C, flags=cv.DFT_SCALE | cv.DFT_REAL_OUTPUT)
        h, w = resp.shape
        _, mval, _, (mx, my) = cv.minMaxLoc(resp)
        side_resp = resp.copy()
        cv.rectangle(side_resp, (mx-5, my-5), (mx+5, my+5), 0, -1)
        smean, sstd = side_resp.mean(), side_resp.std()
        psr = (mval-smean) / (sstd+eps)
        return resp, (mx-w//2, my-h//2), psr

    def update_kernel(self):
        self.H = divSpec(self.H1, self.H2)
        self.H[...,1] *= -1

class App:
    def __init__(self, video_src, paused = False):
        self.cap = video.create_capture(video_src)
        _, self.frame = self.cap.read()
        cv.imshow('frame', self.frame)
        self.rect_sel = RectSelector('frame', self.onrect)
        self.trackers = []
        self.paused = paused

    def onrect(self, rect):
        frame_gray = cv.cvtColor(self.frame, cv.COLOR_BGR2GRAY)
        tracker = MOSSE(frame_gray, rect)
        self.trackers.append(tracker)

    def run(self):
        while True:
            if not self.paused:
                ret, self.frame = self.cap.read()
                if not ret:
                    break
                frame_gray = cv.cvtColor(self.frame, cv.COLOR_BGR2GRAY)
                for tracker in self.trackers:
                    tracker.update(frame_gray)

            vis = self.frame.copy()
            for tracker in self.trackers:
                tracker.draw_state(vis)
            if len(self.trackers) > 0:
                cv.imshow('tracker state', self.trackers[-1].state_vis)
            self.rect_sel.draw(vis)

            cv.imshow('frame', vis)
            ch = cv.waitKey(10)
            if ch == 27:
                break
            if ch == ord(' '):
                self.paused = not self.paused
            if ch == ord('c'):
                self.trackers = []


if __name__ == '__main__':
    print (__doc__)
    import sys, getopt
    opts, args = getopt.getopt(sys.argv[1:], '', ['pause'])
    opts = dict(opts)
    try:
        video_src = args[0]
    except:
        video_src = '0'

    App(video_src, paused = '--pause' in opts).run()



__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


# Classes
class IntelligentScissorsMB:
    # Functions
    def __init__(self) -> None: ...

    def setWeights(self, weight_non_edge: float, weight_gradient_direction: float, weight_gradient_magnitude: float) -> IntelligentScissorsMB: ...

    def setGradientMagnitudeMaxLimit(self, gradient_magnitude_threshold_max: float = ...) -> IntelligentScissorsMB: ...

    def setEdgeFeatureZeroCrossingParameters(self, gradient_magnitude_min_value: float = ...) -> IntelligentScissorsMB: ...

    def setEdgeFeatureCannyParameters(self, threshold1: float, threshold2: float, apertureSize: int = ..., L2gradient: bool = ...) -> IntelligentScissorsMB: ...

    @_typing.overload
    def applyImage(self, image: cv2.typing.MatLike) -> IntelligentScissorsMB: ...
    @_typing.overload
    def applyImage(self, image: cv2.UMat) -> IntelligentScissorsMB: ...

    @_typing.overload
    def applyImageFeatures(self, non_edge: cv2.typing.MatLike, gradient_direction: cv2.typing.MatLike, gradient_magnitude: cv2.typing.MatLike, image: cv2.typing.MatLike | None = ...) -> IntelligentScissorsMB: ...
    @_typing.overload
    def applyImageFeatures(self, non_edge: cv2.UMat, gradient_direction: cv2.UMat, gradient_magnitude: cv2.UMat, image: cv2.UMat | None = ...) -> IntelligentScissorsMB: ...

    def buildMap(self, sourcePt: cv2.typing.Point) -> None: ...

    @_typing.overload
    def getContour(self, targetPt: cv2.typing.Point, contour: cv2.typing.MatLike | None = ..., backward: bool = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getContour(self, targetPt: cv2.typing.Point, contour: cv2.UMat | None = ..., backward: bool = ...) -> cv2.UMat: ...






__all__: list[str] = []

import cv2
import cv2.kinfu
import cv2.typing
import typing as _typing


# Classes
class DynaFu:
    # Functions
    @classmethod
    def create(cls, _params: cv2.kinfu.Params) -> DynaFu: ...

    @_typing.overload
    def render(self, image: cv2.typing.MatLike | None = ..., cameraPose: cv2.typing.Matx44f = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def render(self, image: cv2.UMat | None = ..., cameraPose: cv2.typing.Matx44f = ...) -> cv2.UMat: ...

    @_typing.overload
    def getCloud(self, points: cv2.typing.MatLike | None = ..., normals: cv2.typing.MatLike | None = ...) -> tuple[cv2.typing.MatLike, cv2.typing.MatLike]: ...
    @_typing.overload
    def getCloud(self, points: cv2.UMat | None = ..., normals: cv2.UMat | None = ...) -> tuple[cv2.UMat, cv2.UMat]: ...

    @_typing.overload
    def getPoints(self, points: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getPoints(self, points: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def getNormals(self, points: cv2.typing.MatLike, normals: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getNormals(self, points: cv2.UMat, normals: cv2.UMat | None = ...) -> cv2.UMat: ...

    def reset(self) -> None: ...

    @_typing.overload
    def update(self, depth: cv2.typing.MatLike) -> bool: ...
    @_typing.overload
    def update(self, depth: cv2.UMat) -> bool: ...






__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


# Enumerations
RETINA_COLOR_RANDOM: int
RETINA_COLOR_DIAGONAL: int
RETINA_COLOR_BAYER: int



# Classes
class Retina(cv2.Algorithm):
    # Functions
    def getInputSize(self) -> cv2.typing.Size: ...

    def getOutputSize(self) -> cv2.typing.Size: ...

    def setup(self, retinaParameterFile: str = ..., applyDefaultSetupOnFailure: bool = ...) -> None: ...

    def printSetup(self) -> str: ...

    def write(self, fs: str) -> None: ...

    def setupOPLandIPLParvoChannel(self, colorMode: bool = ..., normaliseOutput: bool = ..., photoreceptorsLocalAdaptationSensitivity: float = ..., photoreceptorsTemporalConstant: float = ..., photoreceptorsSpatialConstant: float = ..., horizontalCellsGain: float = ..., HcellsTemporalConstant: float = ..., HcellsSpatialConstant: float = ..., ganglionCellsSensitivity: float = ...) -> None: ...

    def setupIPLMagnoChannel(self, normaliseOutput: bool = ..., parasolCells_beta: float = ..., parasolCells_tau: float = ..., parasolCells_k: float = ..., amacrinCellsTemporalCutFrequency: float = ..., V0CompressionParameter: float = ..., localAdaptintegration_tau: float = ..., localAdaptintegration_k: float = ...) -> None: ...

    @_typing.overload
    def run(self, inputImage: cv2.typing.MatLike) -> None: ...
    @_typing.overload
    def run(self, inputImage: cv2.UMat) -> None: ...

    @_typing.overload
    def applyFastToneMapping(self, inputImage: cv2.typing.MatLike, outputToneMappedImage: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def applyFastToneMapping(self, inputImage: cv2.UMat, outputToneMappedImage: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def getParvo(self, retinaOutput_parvo: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getParvo(self, retinaOutput_parvo: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def getParvoRAW(self, retinaOutput_parvo: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getParvoRAW(self, retinaOutput_parvo: cv2.UMat | None = ...) -> cv2.UMat: ...
    @_typing.overload
    def getParvoRAW(self) -> cv2.typing.MatLike: ...

    @_typing.overload
    def getMagno(self, retinaOutput_magno: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getMagno(self, retinaOutput_magno: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def getMagnoRAW(self, retinaOutput_magno: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getMagnoRAW(self, retinaOutput_magno: cv2.UMat | None = ...) -> cv2.UMat: ...
    @_typing.overload
    def getMagnoRAW(self) -> cv2.typing.MatLike: ...

    def setColorSaturation(self, saturateColors: bool = ..., colorSaturationValue: float = ...) -> None: ...

    def clearBuffers(self) -> None: ...

    def activateMovingContoursProcessing(self, activate: bool) -> None: ...

    def activateContoursProcessing(self, activate: bool) -> None: ...

    @classmethod
    @_typing.overload
    def create(cls, inputSize: cv2.typing.Size) -> Retina: ...
    @classmethod
    @_typing.overload
    def create(cls, inputSize: cv2.typing.Size, colorMode: bool, colorSamplingMethod: int = ..., useRetinaLogSampling: bool = ..., reductionFactor: float = ..., samplingStrength: float = ...) -> Retina: ...


class RetinaFastToneMapping(cv2.Algorithm):
    # Functions
    @_typing.overload
    def applyFastToneMapping(self, inputImage: cv2.typing.MatLike, outputToneMappedImage: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def applyFastToneMapping(self, inputImage: cv2.UMat, outputToneMappedImage: cv2.UMat | None = ...) -> cv2.UMat: ...

    def setup(self, photoreceptorsNeighborhoodRadius: float = ..., ganglioncellsNeighborhoodRadius: float = ..., meanLuminanceModulatorK: float = ...) -> None: ...

    @classmethod
    def create(cls, inputSize: cv2.typing.Size) -> RetinaFastToneMapping: ...


class TransientAreasSegmentationModule(cv2.Algorithm):
    # Functions
    def getSize(self) -> cv2.typing.Size: ...

    def setup(self, segmentationParameterFile: str = ..., applyDefaultSetupOnFailure: bool = ...) -> None: ...

    def printSetup(self) -> str: ...

    def write(self, fs: str) -> None: ...

    @_typing.overload
    def run(self, inputToSegment: cv2.typing.MatLike, channelIndex: int = ...) -> None: ...
    @_typing.overload
    def run(self, inputToSegment: cv2.UMat, channelIndex: int = ...) -> None: ...

    @_typing.overload
    def getSegmentationPicture(self, transientAreas: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getSegmentationPicture(self, transientAreas: cv2.UMat | None = ...) -> cv2.UMat: ...

    def clearAllBuffers(self) -> None: ...

    @classmethod
    def create(cls, inputSize: cv2.typing.Size) -> TransientAreasSegmentationModule: ...






__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


# Enumerations
FEATURE_SET_COMPUTE_10: int
FEATURE_SET_COMPUTE_11: int
FEATURE_SET_COMPUTE_12: int
FEATURE_SET_COMPUTE_13: int
FEATURE_SET_COMPUTE_20: int
FEATURE_SET_COMPUTE_21: int
FEATURE_SET_COMPUTE_30: int
FEATURE_SET_COMPUTE_32: int
FEATURE_SET_COMPUTE_35: int
FEATURE_SET_COMPUTE_50: int
GLOBAL_ATOMICS: int
SHARED_ATOMICS: int
NATIVE_DOUBLE: int
WARP_SHUFFLE_FUNCTIONS: int
DYNAMIC_PARALLELISM: int
FeatureSet = int
"""One of [FEATURE_SET_COMPUTE_10, FEATURE_SET_COMPUTE_11, FEATURE_SET_COMPUTE_12, FEATURE_SET_COMPUTE_13, FEATURE_SET_COMPUTE_20, FEATURE_SET_COMPUTE_21, FEATURE_SET_COMPUTE_30, FEATURE_SET_COMPUTE_32, FEATURE_SET_COMPUTE_35, FEATURE_SET_COMPUTE_50, GLOBAL_ATOMICS, SHARED_ATOMICS, NATIVE_DOUBLE, WARP_SHUFFLE_FUNCTIONS, DYNAMIC_PARALLELISM]"""


HostMem_PAGE_LOCKED: int
HOST_MEM_PAGE_LOCKED: int
HostMem_SHARED: int
HOST_MEM_SHARED: int
HostMem_WRITE_COMBINED: int
HOST_MEM_WRITE_COMBINED: int
HostMem_AllocType = int
"""One of [HostMem_PAGE_LOCKED, HOST_MEM_PAGE_LOCKED, HostMem_SHARED, HOST_MEM_SHARED, HostMem_WRITE_COMBINED, HOST_MEM_WRITE_COMBINED]"""

Event_DEFAULT: int
EVENT_DEFAULT: int
Event_BLOCKING_SYNC: int
EVENT_BLOCKING_SYNC: int
Event_DISABLE_TIMING: int
EVENT_DISABLE_TIMING: int
Event_INTERPROCESS: int
EVENT_INTERPROCESS: int
Event_CreateFlags = int
"""One of [Event_DEFAULT, EVENT_DEFAULT, Event_BLOCKING_SYNC, EVENT_BLOCKING_SYNC, Event_DISABLE_TIMING, EVENT_DISABLE_TIMING, Event_INTERPROCESS, EVENT_INTERPROCESS]"""

DeviceInfo_ComputeModeDefault: int
DEVICE_INFO_COMPUTE_MODE_DEFAULT: int
DeviceInfo_ComputeModeExclusive: int
DEVICE_INFO_COMPUTE_MODE_EXCLUSIVE: int
DeviceInfo_ComputeModeProhibited: int
DEVICE_INFO_COMPUTE_MODE_PROHIBITED: int
DeviceInfo_ComputeModeExclusiveProcess: int
DEVICE_INFO_COMPUTE_MODE_EXCLUSIVE_PROCESS: int
DeviceInfo_ComputeMode = int
"""One of [DeviceInfo_ComputeModeDefault, DEVICE_INFO_COMPUTE_MODE_DEFAULT, DeviceInfo_ComputeModeExclusive, DEVICE_INFO_COMPUTE_MODE_EXCLUSIVE, DeviceInfo_ComputeModeProhibited, DEVICE_INFO_COMPUTE_MODE_PROHIBITED, DeviceInfo_ComputeModeExclusiveProcess, DEVICE_INFO_COMPUTE_MODE_EXCLUSIVE_PROCESS]"""

SURF_CUDA_X_ROW: int
SURF_CUDA_Y_ROW: int
SURF_CUDA_LAPLACIAN_ROW: int
SURF_CUDA_OCTAVE_ROW: int
SURF_CUDA_SIZE_ROW: int
SURF_CUDA_ANGLE_ROW: int
SURF_CUDA_HESSIAN_ROW: int
SURF_CUDA_ROWS_COUNT: int
SURF_CUDA_KeypointLayout = int
"""One of [SURF_CUDA_X_ROW, SURF_CUDA_Y_ROW, SURF_CUDA_LAPLACIAN_ROW, SURF_CUDA_OCTAVE_ROW, SURF_CUDA_SIZE_ROW, SURF_CUDA_ANGLE_ROW, SURF_CUDA_HESSIAN_ROW, SURF_CUDA_ROWS_COUNT]"""


# Classes
class GpuMat:
    @property
    def step(self) -> int: ...

    # Classes
    class Allocator:
        ...


    # Functions
    @_typing.overload
    def __init__(self, allocator: GpuMat.Allocator = ...) -> None: ...
    @_typing.overload
    def __init__(self, rows: int, cols: int, type: int, allocator: GpuMat.Allocator = ...) -> None: ...
    @_typing.overload
    def __init__(self, size: cv2.typing.Size, type: int, allocator: GpuMat.Allocator = ...) -> None: ...
    @_typing.overload
    def __init__(self, rows: int, cols: int, type: int, s: cv2.typing.Scalar, allocator: GpuMat.Allocator = ...) -> None: ...
    @_typing.overload
    def __init__(self, size: cv2.typing.Size, type: int, s: cv2.typing.Scalar, allocator: GpuMat.Allocator = ...) -> None: ...
    @_typing.overload
    def __init__(self, m: GpuMat) -> None: ...
    @_typing.overload
    def __init__(self, m: GpuMat, rowRange: cv2.typing.Range, colRange: cv2.typing.Range) -> None: ...
    @_typing.overload
    def __init__(self, m: GpuMat, roi: cv2.typing.Rect) -> None: ...
    @_typing.overload
    def __init__(self, arr: cv2.typing.MatLike, allocator: GpuMat.Allocator = ...) -> None: ...
    @_typing.overload
    def __init__(self, arr: GpuMat, allocator: GpuMat.Allocator = ...) -> None: ...
    @_typing.overload
    def __init__(self, arr: cv2.UMat, allocator: GpuMat.Allocator = ...) -> None: ...

    @staticmethod
    def defaultAllocator() -> GpuMat.Allocator: ...

    @staticmethod
    def setDefaultAllocator(allocator: GpuMat.Allocator) -> None: ...

    @staticmethod
    def getStdAllocator() -> GpuMat.Allocator: ...

    @_typing.overload
    def create(self, rows: int, cols: int, type: int) -> None: ...
    @_typing.overload
    def create(self, size: cv2.typing.Size, type: int) -> None: ...

    def release(self) -> None: ...

    def swap(self, mat: GpuMat) -> None: ...

    @_typing.overload
    def upload(self, arr: cv2.typing.MatLike) -> None: ...
    @_typing.overload
    def upload(self, arr: GpuMat) -> None: ...
    @_typing.overload
    def upload(self, arr: cv2.UMat) -> None: ...
    @_typing.overload
    def upload(self, arr: cv2.typing.MatLike, stream: Stream) -> None: ...
    @_typing.overload
    def upload(self, arr: GpuMat, stream: Stream) -> None: ...
    @_typing.overload
    def upload(self, arr: cv2.UMat, stream: Stream) -> None: ...

    @_typing.overload
    def download(self, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def download(self, dst: GpuMat | None = ...) -> GpuMat: ...
    @_typing.overload
    def download(self, dst: cv2.UMat | None = ...) -> cv2.UMat: ...
    @_typing.overload
    def download(self, stream: Stream, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def download(self, stream: Stream, dst: GpuMat | None = ...) -> GpuMat: ...
    @_typing.overload
    def download(self, stream: Stream, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

    def clone(self) -> GpuMat: ...

    @_typing.overload
    def copyTo(self, dst: GpuMat | None = ...) -> GpuMat: ...
    @_typing.overload
    def copyTo(self, stream: Stream, dst: GpuMat | None = ...) -> GpuMat: ...
    @_typing.overload
    def copyTo(self, mask: GpuMat, dst: GpuMat | None = ...) -> GpuMat: ...
    @_typing.overload
    def copyTo(self, mask: GpuMat, stream: Stream, dst: GpuMat | None = ...) -> GpuMat: ...

    @_typing.overload
    def setTo(self, s: cv2.typing.Scalar) -> GpuMat: ...
    @_typing.overload
    def setTo(self, s: cv2.typing.Scalar, stream: Stream) -> GpuMat: ...
    @_typing.overload
    def setTo(self, s: cv2.typing.Scalar, mask: cv2.typing.MatLike) -> GpuMat: ...
    @_typing.overload
    def setTo(self, s: cv2.typing.Scalar, mask: GpuMat) -> GpuMat: ...
    @_typing.overload
    def setTo(self, s: cv2.typing.Scalar, mask: cv2.UMat) -> GpuMat: ...
    @_typing.overload
    def setTo(self, s: cv2.typing.Scalar, mask: cv2.typing.MatLike, stream: Stream) -> GpuMat: ...
    @_typing.overload
    def setTo(self, s: cv2.typing.Scalar, mask: GpuMat, stream: Stream) -> GpuMat: ...
    @_typing.overload
    def setTo(self, s: cv2.typing.Scalar, mask: cv2.UMat, stream: Stream) -> GpuMat: ...

    @_typing.overload
    def convertTo(self, rtype: int, dst: GpuMat | None = ...) -> GpuMat: ...
    @_typing.overload
    def convertTo(self, rtype: int, stream: Stream, dst: GpuMat | None = ...) -> GpuMat: ...
    @_typing.overload
    def convertTo(self, rtype: int, dst: GpuMat | None = ..., alpha: float = ..., beta: float = ...) -> GpuMat: ...
    @_typing.overload
    def convertTo(self, rtype: int, alpha: float, beta: float, stream: Stream, dst: GpuMat | None = ...) -> GpuMat: ...

    def assignTo(self, m: GpuMat, type: int = ...) -> None: ...

    def row(self, y: int) -> GpuMat: ...

    def col(self, x: int) -> GpuMat: ...

    @_typing.overload
    def rowRange(self, startrow: int, endrow: int) -> GpuMat: ...
    @_typing.overload
    def rowRange(self, r: cv2.typing.Range) -> GpuMat: ...

    @_typing.overload
    def colRange(self, startcol: int, endcol: int) -> GpuMat: ...
    @_typing.overload
    def colRange(self, r: cv2.typing.Range) -> GpuMat: ...

    def reshape(self, cn: int, rows: int = ...) -> GpuMat: ...

    def locateROI(self, wholeSize: cv2.typing.Size, ofs: cv2.typing.Point) -> None: ...

    def adjustROI(self, dtop: int, dbottom: int, dleft: int, dright: int) -> GpuMat: ...

    def isContinuous(self) -> bool: ...

    def elemSize(self) -> int: ...

    def elemSize1(self) -> int: ...

    def type(self) -> int: ...

    def depth(self) -> int: ...

    def channels(self) -> int: ...

    def step1(self) -> int: ...

    def size(self) -> cv2.typing.Size: ...

    def empty(self) -> bool: ...

    def cudaPtr(self) -> cv2.typing.IntPointer: ...

    def updateContinuityFlag(self) -> None: ...


class GpuData:
    ...

class GpuMatND:
    ...

class BufferPool:
    # Functions
    def __init__(self, stream: Stream) -> None: ...

    @_typing.overload
    def getBuffer(self, rows: int, cols: int, type: int) -> GpuMat: ...
    @_typing.overload
    def getBuffer(self, size: cv2.typing.Size, type: int) -> GpuMat: ...

    def getAllocator(self) -> GpuMat.Allocator: ...


class HostMem:
    @property
    def step(self) -> int: ...

    # Functions
    @_typing.overload
    def __init__(self, alloc_type: HostMem_AllocType = ...) -> None: ...
    @_typing.overload
    def __init__(self, rows: int, cols: int, type: int, alloc_type: HostMem_AllocType = ...) -> None: ...
    @_typing.overload
    def __init__(self, size: cv2.typing.Size, type: int, alloc_type: HostMem_AllocType = ...) -> None: ...
    @_typing.overload
    def __init__(self, arr: cv2.typing.MatLike, alloc_type: HostMem_AllocType = ...) -> None: ...
    @_typing.overload
    def __init__(self, arr: GpuMat, alloc_type: HostMem_AllocType = ...) -> None: ...
    @_typing.overload
    def __init__(self, arr: cv2.UMat, alloc_type: HostMem_AllocType = ...) -> None: ...

    def swap(self, b: HostMem) -> None: ...

    def clone(self) -> HostMem: ...

    def create(self, rows: int, cols: int, type: int) -> None: ...

    def reshape(self, cn: int, rows: int = ...) -> HostMem: ...

    def createMatHeader(self) -> cv2.typing.MatLike: ...

    def isContinuous(self) -> bool: ...

    def elemSize(self) -> int: ...

    def elemSize1(self) -> int: ...

    def type(self) -> int: ...

    def depth(self) -> int: ...

    def channels(self) -> int: ...

    def step1(self) -> int: ...

    def size(self) -> cv2.typing.Size: ...

    def empty(self) -> bool: ...


class Stream:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, allocator: GpuMat.Allocator) -> None: ...
    @_typing.overload
    def __init__(self, cudaFlags: int) -> None: ...

    def queryIfComplete(self) -> bool: ...

    def waitForCompletion(self) -> None: ...

    def waitEvent(self, event: Event) -> None: ...

    @classmethod
    def Null(cls) -> Stream: ...

    def cudaPtr(self) -> cv2.typing.IntPointer: ...


class Event:
    # Functions
    def __init__(self, flags: Event_CreateFlags = ...) -> None: ...

    def record(self, stream: Stream = ...) -> None: ...

    def queryIfComplete(self) -> bool: ...

    def waitForCompletion(self) -> None: ...

    @staticmethod
    def elapsedTime(start: Event, end: Event) -> float: ...


class TargetArchs:
    # Functions
    @staticmethod
    def has(major: int, minor: int) -> bool: ...

    @staticmethod
    def hasPtx(major: int, minor: int) -> bool: ...

    @staticmethod
    def hasBin(major: int, minor: int) -> bool: ...

    @staticmethod
    def hasEqualOrLessPtx(major: int, minor: int) -> bool: ...

    @staticmethod
    def hasEqualOrGreater(major: int, minor: int) -> bool: ...

    @staticmethod
    def hasEqualOrGreaterPtx(major: int, minor: int) -> bool: ...

    @staticmethod
    def hasEqualOrGreaterBin(major: int, minor: int) -> bool: ...


class DeviceInfo:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, device_id: int) -> None: ...

    def deviceID(self) -> int: ...

    def totalGlobalMem(self) -> int: ...

    def sharedMemPerBlock(self) -> int: ...

    def regsPerBlock(self) -> int: ...

    def warpSize(self) -> int: ...

    def memPitch(self) -> int: ...

    def maxThreadsPerBlock(self) -> int: ...

    def maxThreadsDim(self) -> cv2.typing.Vec3i: ...

    def maxGridSize(self) -> cv2.typing.Vec3i: ...

    def clockRate(self) -> int: ...

    def totalConstMem(self) -> int: ...

    def majorVersion(self) -> int: ...

    def minorVersion(self) -> int: ...

    def textureAlignment(self) -> int: ...

    def texturePitchAlignment(self) -> int: ...

    def multiProcessorCount(self) -> int: ...

    def kernelExecTimeoutEnabled(self) -> bool: ...

    def integrated(self) -> bool: ...

    def canMapHostMemory(self) -> bool: ...

    def computeMode(self) -> DeviceInfo_ComputeMode: ...

    def maxTexture1D(self) -> int: ...

    def maxTexture1DMipmap(self) -> int: ...

    def maxTexture1DLinear(self) -> int: ...

    def maxTexture2D(self) -> cv2.typing.Vec2i: ...

    def maxTexture2DMipmap(self) -> cv2.typing.Vec2i: ...

    def maxTexture2DLinear(self) -> cv2.typing.Vec3i: ...

    def maxTexture2DGather(self) -> cv2.typing.Vec2i: ...

    def maxTexture3D(self) -> cv2.typing.Vec3i: ...

    def maxTextureCubemap(self) -> int: ...

    def maxTexture1DLayered(self) -> cv2.typing.Vec2i: ...

    def maxTexture2DLayered(self) -> cv2.typing.Vec3i: ...

    def maxTextureCubemapLayered(self) -> cv2.typing.Vec2i: ...

    def maxSurface1D(self) -> int: ...

    def maxSurface2D(self) -> cv2.typing.Vec2i: ...

    def maxSurface3D(self) -> cv2.typing.Vec3i: ...

    def maxSurface1DLayered(self) -> cv2.typing.Vec2i: ...

    def maxSurface2DLayered(self) -> cv2.typing.Vec3i: ...

    def maxSurfaceCubemap(self) -> int: ...

    def maxSurfaceCubemapLayered(self) -> cv2.typing.Vec2i: ...

    def surfaceAlignment(self) -> int: ...

    def concurrentKernels(self) -> bool: ...

    def ECCEnabled(self) -> bool: ...

    def pciBusID(self) -> int: ...

    def pciDeviceID(self) -> int: ...

    def pciDomainID(self) -> int: ...

    def tccDriver(self) -> bool: ...

    def asyncEngineCount(self) -> int: ...

    def unifiedAddressing(self) -> bool: ...

    def memoryClockRate(self) -> int: ...

    def memoryBusWidth(self) -> int: ...

    def l2CacheSize(self) -> int: ...

    def maxThreadsPerMultiProcessor(self) -> int: ...

    def queryMemory(self, totalMemory: int, freeMemory: int) -> None: ...

    def freeMemory(self) -> int: ...

    def totalMemory(self) -> int: ...

    def isCompatible(self) -> bool: ...


class SURF_CUDA:
    @property
    def hessianThreshold(self) -> float: ...
    @property
    def nOctaves(self) -> int: ...
    @property
    def nOctaveLayers(self) -> int: ...
    @property
    def extended(self) -> bool: ...
    @property
    def upright(self) -> bool: ...
    @property
    def keypointsRatio(self) -> float: ...

    # Functions
    @classmethod
    def create(cls, _hessianThreshold: float, _nOctaves: int = ..., _nOctaveLayers: int = ..., _extended: bool = ..., _keypointsRatio: float = ..., _upright: bool = ...) -> SURF_CUDA: ...

    def descriptorSize(self) -> int: ...

    def defaultNorm(self) -> int: ...

    def downloadKeypoints(self, keypointsGPU: GpuMat) -> _typing.Sequence[cv2.KeyPoint]: ...

    def detect(self, img: GpuMat, mask: GpuMat, keypoints: GpuMat | None = ...) -> GpuMat: ...

    def detectWithDescriptors(self, img: GpuMat, mask: GpuMat, keypoints: GpuMat | None = ..., descriptors: GpuMat | None = ..., useProvidedKeypoints: bool = ...) -> tuple[GpuMat, GpuMat]: ...



# Functions
@_typing.overload
def createContinuous(rows: int, cols: int, type: int, arr: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def createContinuous(rows: int, cols: int, type: int, arr: GpuMat | None = ...) -> GpuMat: ...
@_typing.overload
def createContinuous(rows: int, cols: int, type: int, arr: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def createGpuMatFromCudaMemory(rows: int, cols: int, type: int, cudaMemoryAddress: int, step: int = ...) -> GpuMat: ...
@_typing.overload
def createGpuMatFromCudaMemory(size: cv2.typing.Size, type: int, cudaMemoryAddress: int, step: int = ...) -> GpuMat: ...

@_typing.overload
def ensureSizeIsEnough(rows: int, cols: int, type: int, arr: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def ensureSizeIsEnough(rows: int, cols: int, type: int, arr: GpuMat | None = ...) -> GpuMat: ...
@_typing.overload
def ensureSizeIsEnough(rows: int, cols: int, type: int, arr: cv2.UMat | None = ...) -> cv2.UMat: ...

def fastNlMeansDenoising(src: GpuMat, h: float, dst: GpuMat | None = ..., search_window: int = ..., block_size: int = ..., stream: Stream = ...) -> GpuMat: ...

def fastNlMeansDenoisingColored(src: GpuMat, h_luminance: float, photo_render: float, dst: GpuMat | None = ..., search_window: int = ..., block_size: int = ..., stream: Stream = ...) -> GpuMat: ...

def getCudaEnabledDeviceCount() -> int: ...

def getDevice() -> int: ...

def nonLocalMeans(src: GpuMat, h: float, dst: GpuMat | None = ..., search_window: int = ..., block_size: int = ..., borderMode: int = ..., stream: Stream = ...) -> GpuMat: ...

def printCudaDeviceInfo(device: int) -> None: ...

def printShortCudaDeviceInfo(device: int) -> None: ...

def registerPageLocked(m: cv2.typing.MatLike) -> None: ...

def resetDevice() -> None: ...

def setBufferPoolConfig(deviceId: int, stackSize: int, stackCount: int) -> None: ...

def setBufferPoolUsage(on: bool) -> None: ...

def setDevice(device: int) -> None: ...

def unregisterPageLocked(m: cv2.typing.MatLike) -> None: ...

def wrapStream(cudaStreamMemoryAddress: int) -> Stream: ...





__all__: list[str] = []

import cv2.typing
import os
import typing as _typing


# Classes
class ICP:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, iterations: int, tolerence: float = ..., rejectionScale: float = ..., numLevels: int = ..., sampleType: int = ..., numMaxCorr: int = ...) -> None: ...

    @_typing.overload
    def registerModelToScene(self, srcPC: cv2.typing.MatLike, dstPC: cv2.typing.MatLike) -> tuple[int, float, cv2.typing.Matx44d]: ...
    @_typing.overload
    def registerModelToScene(self, srcPC: cv2.typing.MatLike, dstPC: cv2.typing.MatLike, poses: _typing.Sequence[Pose3D]) -> tuple[int, _typing.Sequence[Pose3D]]: ...


class Pose3D:
    @property
    def alpha(self) -> float: ...
    @property
    def residual(self) -> float: ...
    @property
    def modelIndex(self) -> int: ...
    @property
    def numVotes(self) -> int: ...
    @property
    def pose(self) -> cv2.typing.Matx44d: ...
    @property
    def angle(self) -> float: ...
    @property
    def t(self) -> cv2.typing.Vec3d: ...
    @property
    def q(self) -> cv2.typing.Vec4d: ...

    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, Alpha: float, ModelIndex: int = ..., NumVotes: int = ...) -> None: ...

    @_typing.overload
    def updatePose(self, NewPose: cv2.typing.Matx44d) -> None: ...
    @_typing.overload
    def updatePose(self, NewR: cv2.typing.Matx33d, NewT: cv2.typing.Vec3d) -> None: ...

    def updatePoseQuat(self, Q: cv2.typing.Vec4d, NewT: cv2.typing.Vec3d) -> None: ...

    def appendPose(self, IncrementalPose: cv2.typing.Matx44d) -> None: ...

    def printPose(self) -> None: ...


class PoseCluster3D:
    ...

class PPF3DDetector:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, relativeSamplingStep: float, relativeDistanceStep: float = ..., numAngles: float = ...) -> None: ...

    def trainModel(self, Model: cv2.typing.MatLike) -> None: ...

    def match(self, scene: cv2.typing.MatLike, relativeSceneSampleStep: float = ..., relativeSceneDistance: float = ...) -> _typing.Sequence[Pose3D]: ...



# Functions
def addNoisePC(pc: cv2.typing.MatLike, scale: float) -> cv2.typing.MatLike: ...

def computeNormalsPC3d(PC: cv2.typing.MatLike, NumNeighbors: int, FlipViewpoint: bool, viewpoint: cv2.typing.Vec3f, PCNormals: cv2.typing.MatLike | None = ...) -> tuple[int, cv2.typing.MatLike]: ...

def getRandomPose(Pose: cv2.typing.Matx44d) -> None: ...

def loadPLYSimple(fileName: str | os.PathLike[str], withNormals: int = ...) -> cv2.typing.MatLike: ...

def samplePCByQuantization(pc: cv2.typing.MatLike, xrange: cv2.typing.Vec2f, yrange: cv2.typing.Vec2f, zrange: cv2.typing.Vec2f, sample_step_relative: float, weightByCenter: int = ...) -> cv2.typing.MatLike: ...

def transformPCPose(pc: cv2.typing.MatLike, Pose: cv2.typing.Matx44d) -> cv2.typing.MatLike: ...

def writePLY(PC: cv2.typing.MatLike, fileName: str | os.PathLike[str]) -> None: ...

def writePLYVisibleNormals(PC: cv2.typing.MatLike, fileName: str | os.PathLike[str]) -> None: ...





__all__: list[str] = []

# Enumerations
TYPE_GENERAL: int
TYPE_MARKER: int
TYPE_WRAPPER: int
TYPE_FUN: int
TYPE = int
"""One of [TYPE_GENERAL, TYPE_MARKER, TYPE_WRAPPER, TYPE_FUN]"""

IMPL_PLAIN: int
IMPL_IPP: int
IMPL_OPENCL: int
IMPL = int
"""One of [IMPL_PLAIN, IMPL_IPP, IMPL_OPENCL]"""

FLAGS_NONE: int
FLAGS_MAPPING: int
FLAGS_EXPAND_SAME_NAMES: int
FLAGS = int
"""One of [FLAGS_NONE, FLAGS_MAPPING, FLAGS_EXPAND_SAME_NAMES]"""






__all__: list[str] = []

import cv2.gapi.onnx.ep
import cv2.typing
import typing as _typing


from cv2.gapi.onnx import ep as ep


# Enumerations
TraitAs_TENSOR: int
TRAIT_AS_TENSOR: int
TraitAs_IMAGE: int
TRAIT_AS_IMAGE: int
TraitAs = int
"""One of [TraitAs_TENSOR, TRAIT_AS_TENSOR, TraitAs_IMAGE, TRAIT_AS_IMAGE]"""



# Classes
class PyParams:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, tag: str, model_path: str) -> None: ...

    def cfgMeanStd(self, layer_name: str, m: cv2.typing.Scalar, s: cv2.typing.Scalar) -> PyParams: ...

    def cfgNormalize(self, layer_name: str, flag: bool) -> PyParams: ...

    @_typing.overload
    def cfgAddExecutionProvider(self, ep: cv2.gapi.onnx.ep.OpenVINO) -> PyParams: ...
    @_typing.overload
    def cfgAddExecutionProvider(self, ep: cv2.gapi.onnx.ep.DirectML) -> PyParams: ...
    @_typing.overload
    def cfgAddExecutionProvider(self, ep: cv2.gapi.onnx.ep.CoreML) -> PyParams: ...
    @_typing.overload
    def cfgAddExecutionProvider(self, ep: cv2.gapi.onnx.ep.CUDA) -> PyParams: ...
    @_typing.overload
    def cfgAddExecutionProvider(self, ep: cv2.gapi.onnx.ep.TensorRT) -> PyParams: ...

    def cfgDisableMemPattern(self) -> PyParams: ...

    def cfgSessionOptions(self, options: cv2.typing.map_string_and_string) -> PyParams: ...

    def cfgOptLevel(self, opt_level: int) -> PyParams: ...



# Functions
def params(tag: str, model_path: str) -> PyParams: ...





__all__: list[str] = []

import cv2
import cv2.typing
import os
import typing as _typing


# Enumerations
FLANN_INDEX_TYPE_8U: int
FLANN_INDEX_TYPE_8S: int
FLANN_INDEX_TYPE_16U: int
FLANN_INDEX_TYPE_16S: int
FLANN_INDEX_TYPE_32S: int
FLANN_INDEX_TYPE_32F: int
FLANN_INDEX_TYPE_64F: int
FLANN_INDEX_TYPE_STRING: int
FLANN_INDEX_TYPE_BOOL: int
FLANN_INDEX_TYPE_ALGORITHM: int
LAST_VALUE_FLANN_INDEX_TYPE: int
FlannIndexType = int
"""One of [FLANN_INDEX_TYPE_8U, FLANN_INDEX_TYPE_8S, FLANN_INDEX_TYPE_16U, FLANN_INDEX_TYPE_16S, FLANN_INDEX_TYPE_32S, FLANN_INDEX_TYPE_32F, FLANN_INDEX_TYPE_64F, FLANN_INDEX_TYPE_STRING, FLANN_INDEX_TYPE_BOOL, FLANN_INDEX_TYPE_ALGORITHM, LAST_VALUE_FLANN_INDEX_TYPE]"""



# Classes
class Index:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, features: cv2.typing.MatLike, params: cv2.typing.IndexParams, distType: int = ...) -> None: ...
    @_typing.overload
    def __init__(self, features: cv2.UMat, params: cv2.typing.IndexParams, distType: int = ...) -> None: ...

    @_typing.overload
    def build(self, features: cv2.typing.MatLike, params: cv2.typing.IndexParams, distType: int = ...) -> None: ...
    @_typing.overload
    def build(self, features: cv2.UMat, params: cv2.typing.IndexParams, distType: int = ...) -> None: ...

    @_typing.overload
    def knnSearch(self, query: cv2.typing.MatLike, knn: int, indices: cv2.typing.MatLike | None = ..., dists: cv2.typing.MatLike | None = ..., params: cv2.typing.SearchParams = ...) -> tuple[cv2.typing.MatLike, cv2.typing.MatLike]: ...
    @_typing.overload
    def knnSearch(self, query: cv2.UMat, knn: int, indices: cv2.UMat | None = ..., dists: cv2.UMat | None = ..., params: cv2.typing.SearchParams = ...) -> tuple[cv2.UMat, cv2.UMat]: ...

    @_typing.overload
    def radiusSearch(self, query: cv2.typing.MatLike, radius: float, maxResults: int, indices: cv2.typing.MatLike | None = ..., dists: cv2.typing.MatLike | None = ..., params: cv2.typing.SearchParams = ...) -> tuple[int, cv2.typing.MatLike, cv2.typing.MatLike]: ...
    @_typing.overload
    def radiusSearch(self, query: cv2.UMat, radius: float, maxResults: int, indices: cv2.UMat | None = ..., dists: cv2.UMat | None = ..., params: cv2.typing.SearchParams = ...) -> tuple[int, cv2.UMat, cv2.UMat]: ...

    def save(self, filename: str | os.PathLike[str]) -> None: ...

    @_typing.overload
    def load(self, features: cv2.typing.MatLike, filename: str | os.PathLike[str]) -> bool: ...
    @_typing.overload
    def load(self, features: cv2.UMat, filename: str | os.PathLike[str]) -> bool: ...

    def release(self) -> None: ...

    def getDistance(self) -> int: ...

    def getAlgorithm(self) -> int: ...






__all__: list[str] = []

ParamDesc_Kind_Load: int
PARAM_DESC_KIND_LOAD: int
ParamDesc_Kind_Import: int
PARAM_DESC_KIND_IMPORT: int
ParamDesc_Kind = int
"""One of [ParamDesc_Kind_Load, PARAM_DESC_KIND_LOAD, ParamDesc_Kind_Import, PARAM_DESC_KIND_IMPORT]"""


# Classes




__all__: list[str] = []

# Enumerations
MM_TRANSLATION: int
MM_TRANSLATION_AND_SCALE: int
MM_ROTATION: int
MM_RIGID: int
MM_SIMILARITY: int
MM_AFFINE: int
MM_HOMOGRAPHY: int
MM_UNKNOWN: int
MotionModel = int
"""One of [MM_TRANSLATION, MM_TRANSLATION_AND_SCALE, MM_ROTATION, MM_RIGID, MM_SIMILARITY, MM_AFFINE, MM_HOMOGRAPHY, MM_UNKNOWN]"""






import numpy as np
import cv2 as cv
import sys

if len(sys.argv) != 2:
    print('Input video name is missing')
    exit()

cv.namedWindow("tracking")
camera = cv.VideoCapture(sys.argv[1])
ok, image=camera.read()
if not ok:
    print('Failed to read video')
    exit()
bbox = cv.selectROI("tracking", image)
tracker = cv.TrackerMIL_create()
init_once = False

while camera.isOpened():
    ok, image=camera.read()
    if not ok:
        print('no image to read')
        break

    if not init_once:
        ok = tracker.init(image, bbox)
        init_once = True

    ok, newbox = tracker.update(image)
    print(ok, newbox)

    if ok:
        p1 = (int(newbox[0]), int(newbox[1]))
        p2 = (int(newbox[0] + newbox[2]), int(newbox[1] + newbox[3]))
        cv.rectangle(image, p1, p2, (200,0,0))

    cv.imshow("tracking", image)
    k = cv.waitKey(1) & 0xff
    if k == 27 : break # esc pressed



#!/usr/bin/python

"""

/*********************************************************************
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2016
 *
 * Balint Cristian <cristian dot balint at gmail dot com>
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************/

/* export-boostdesc.py */
/* Export C headers from binary data */
// [http://infoscience.epfl.ch/record/186246/files/boostDesc_1.0.tar.gz]

"""

import sys
import struct



def float_to_hex(f):
  return struct.unpack( '<I', struct.pack('<f', f) )[0]

def main():

  # usage
  if ( len(sys.argv) < 3 ):
    print( "Usage: %s <binary-type (BGM, LBGM, BINBOOST)> <boostdesc-binary-filename>" % sys.argv[0] )
    sys.exit(0)


  if ( ( sys.argv[1] != "BGM" ) and
       ( sys.argv[1] != "LBGM" ) and
       ( sys.argv[1] != "BINBOOST" ) ):
    print( "Invalid type [%s]" % sys.argv[1] )
    sys.exit(0)

  # enum literals
  Assign = [ "ASSIGN_HARD",
             "ASSIGN_BILINEAR",
             "ASSIGN_SOFT",
             "ASSIGN_HARD_MAGN",
             "ASSIGN_SOFT_MAGN" ]

  # open binary data file
  f = open( sys.argv[2], 'rb' )

  # header
  print "/*"
  print " *"
  print " * Header exported from binary."
  print " * [%s %s %s]" % ( sys.argv[0], sys.argv[1], sys.argv[2] )
  print " *"
  print " */"

  # ini
  nDim = 1;
  nWLs = 0;

  # dimensionality (where is the case)
  if ( ( sys.argv[1] == "LBGM" ) or
       ( sys.argv[1] == "BINBOOST" ) ):
    nDim = struct.unpack( '<i', f.read(4) )[0]

  print
  print "// dimensionality of learner"
  print "static const int nDim = %i;" % nDim

  # week learners (where is the case)
  if ( sys.argv[1] != "BINBOOST" ):
    nWLs = struct.unpack( '<i', f.read(4) )[0]

  # common header
  orientQuant     = struct.unpack( '<i', f.read(4) )[0]
  patchSize       = struct.unpack( '<i', f.read(4) )[0]
  iGradAssignType = struct.unpack( '<i', f.read(4) )[0]

  print
  print "// orientations"
  print "static const int orientQuant = %i;" % orientQuant

  print
  print "// patch size"
  print "static const int patchSize = %i;" % patchSize

  print
  print "// gradient assignment type"
  print "static const int iGradAssignType = %s;" % Assign[iGradAssignType]

  arr_thresh = ""
  arr_orient = ""

  arr__y_min = ""
  arr__y_max = ""
  arr__x_min = ""
  arr__x_max = ""

  arr__alpha = ""
  arr___beta = ""


  dims = nDim
  if ( sys.argv[1] == "LBGM" ):
    dims = 1

  # iterate each dimension
  for d in range( 0, dims ):

    if ( sys.argv[1] == "BINBOOST" ):
      nWLs   = struct.unpack( '<i', f.read(4) )[0]

    if ( d == 0 ):
      print
      print "// number of weak learners"
      print "static const int nWLs = %i;" % nWLs

    # iterate each members
    for i in range( 0, nWLs ):

      # unpack structure array
      thresh = struct.unpack( '<f', f.read(4) )[0]
      orient = struct.unpack( '<i', f.read(4) )[0]

      y_min  = struct.unpack( '<i', f.read(4) )[0]
      y_max  = struct.unpack( '<i', f.read(4) )[0]
      x_min  = struct.unpack( '<i', f.read(4) )[0]
      x_max  = struct.unpack( '<i', f.read(4) )[0]

      alpha  = struct.unpack( '<f', f.read(4) )[0]

      beta = 0
      if ( sys.argv[1] == "BINBOOST" ):
        beta = struct.unpack( '<f', f.read(4) )[0]

      # first entry
      if ( d*dims + i == 0 ):

        arr_thresh += "\n"
        arr_thresh += "// threshold array (%s x %s)\n" % (dims,nWLs)
        arr_thresh += "static const unsigned int thresh[] =\n{\n"

        arr_orient += "\n"
        arr_orient += "// orientation array (%s x %s)\n" % (dims,nWLs)
        arr_orient += "static const int orient[] =\n{\n"

        arr__y_min += "\n"
        arr__y_min += "// Y min array (%s x %s)\n" % (dims,nWLs)
        arr__y_min += "static const int y_min[] =\n{\n"

        arr__y_max += "\n"
        arr__y_max += "// Y max array (%s x %s)\n" % (dims,nWLs)
        arr__y_max += "static const int y_max[] =\n{\n"

        arr__x_min += "\n"
        arr__x_min += "// X min array (%s x %s)\n" % (dims,nWLs)
        arr__x_min += "static const int x_min[] =\n{\n"

        arr__x_max += "\n"
        arr__x_max += "// X max array (%s x %s)\n" % (dims,nWLs)
        arr__x_max += "static const int x_max[] =\n{\n"

        arr__alpha += "\n"
        arr__alpha += "// alpha array (%s x %s)\n" % (dims,nWLs)
        arr__alpha += "static const unsigned int alpha[] =\n{\n"

        if ( sys.argv[1] == "BINBOOST" ):
          arr___beta += "\n"
          arr___beta += "// beta array (%s x %s)\n" % (dims,nWLs)
          arr___beta += "static const unsigned int beta[] =\n{\n"

      # last entry
      if ( i == nWLs - 1 ) and ( d == dims - 1):

        arr_thresh += " 0x%08x\n};" % float_to_hex(thresh)
        arr_orient += " 0x%02x\n};" % orient

        arr__y_min += " 0x%02x\n};" % y_min
        arr__y_max += " 0x%02x\n};" % y_max
        arr__x_min += " 0x%02x\n};" % x_min
        arr__x_max += " 0x%02x\n};" % x_max

        arr__alpha += " 0x%08x\n};" % float_to_hex(alpha)

        if ( sys.argv[1] == "BINBOOST" ):
          arr___beta += " 0x%08x\n};" % float_to_hex(beta)

        break

      # align entries
      if ( (d*dims + i + 1) % 8 ):

        arr_thresh += " 0x%08x," % float_to_hex(thresh)
        arr_orient += " 0x%02x," % orient

        arr__y_min += " 0x%02x," % y_min
        arr__y_max += " 0x%02x," % y_max
        arr__x_min += " 0x%02x," % x_min
        arr__x_max += " 0x%02x," % x_max

        arr__alpha += " 0x%08x," % float_to_hex(alpha)

        if ( sys.argv[1] == "BINBOOST" ):
          arr___beta += " 0x%08x," % float_to_hex(beta)

      else:

        arr_thresh += " 0x%08x,\n" % float_to_hex(thresh)
        arr_orient += " 0x%02x,\n" % orient

        arr__y_min += " 0x%02x,\n" % y_min
        arr__y_max += " 0x%02x,\n" % y_max
        arr__x_min += " 0x%02x,\n" % x_min
        arr__x_max += " 0x%02x,\n" % x_max

        arr__alpha += " 0x%08x,\n" % float_to_hex(alpha)

        if ( sys.argv[1] == "BINBOOST" ):
          arr___beta += " 0x%08x,\n" % float_to_hex(beta)

  # extra array (when LBGM)
  if ( sys.argv[1] == "LBGM" ):

    arr___beta += "\n"
    arr___beta += "// beta array (%s x %s)\n" % (nWLs,nDim)
    arr___beta += "static const unsigned int beta[] =\n{\n"

    for i in range( 0, nWLs ):
      for d in range( 0, nDim ):
        beta = struct.unpack( '<f', f.read(4) )[0]

        # last entry
        if ( i == nWLs-1 ) and ( d == nDim-1 ):
          arr___beta += " 0x%08x\n};" % float_to_hex(beta)
          break

        # align entries
        if ( (i*nDim + d + 1) % 8 ):
          arr___beta += " 0x%08x," % float_to_hex(beta)
        else:
          arr___beta += " 0x%08x,\n" % float_to_hex(beta)

  # release
  f.close()

  # dump on screen
  print arr_thresh
  print arr_orient

  print arr__y_min
  print arr__y_max
  print arr__x_min
  print arr__x_max

  print arr__alpha

  if ( ( sys.argv[1] == "LBGM" ) or
       ( sys.argv[1] == "BINBOOST" ) ):
    print arr___beta


if __name__ == "__main__":
    main()



__all__: list[str] = []

import cv2
import cv2.typing
import os
import typing as _typing


# Classes
class FaceRecognizer(cv2.Algorithm):
    # Functions
    @_typing.overload
    def train(self, src: _typing.Sequence[cv2.typing.MatLike], labels: cv2.typing.MatLike) -> None: ...
    @_typing.overload
    def train(self, src: _typing.Sequence[cv2.UMat], labels: cv2.UMat) -> None: ...

    @_typing.overload
    def update(self, src: _typing.Sequence[cv2.typing.MatLike], labels: cv2.typing.MatLike) -> None: ...
    @_typing.overload
    def update(self, src: _typing.Sequence[cv2.UMat], labels: cv2.UMat) -> None: ...

    @_typing.overload
    def predict_label(self, src: cv2.typing.MatLike) -> int: ...
    @_typing.overload
    def predict_label(self, src: cv2.UMat) -> int: ...

    @_typing.overload
    def predict(self, src: cv2.typing.MatLike) -> tuple[int, float]: ...
    @_typing.overload
    def predict(self, src: cv2.UMat) -> tuple[int, float]: ...

    @_typing.overload
    def predict_collect(self, src: cv2.typing.MatLike, collector: PredictCollector) -> None: ...
    @_typing.overload
    def predict_collect(self, src: cv2.UMat, collector: PredictCollector) -> None: ...

    def write(self, filename: str | os.PathLike[str]) -> None: ...

    def read(self, filename: str | os.PathLike[str]) -> None: ...

    def setLabelInfo(self, label: int, strInfo: str) -> None: ...

    def getLabelInfo(self, label: int) -> str: ...

    def getLabelsByString(self, str: str) -> _typing.Sequence[int]: ...


class BIF(cv2.Algorithm):
    # Functions
    def getNumBands(self) -> int: ...

    def getNumRotations(self) -> int: ...

    @_typing.overload
    def compute(self, image: cv2.typing.MatLike, features: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def compute(self, image: cv2.UMat, features: cv2.UMat | None = ...) -> cv2.UMat: ...

    @classmethod
    def create(cls, num_bands: int = ..., num_rotations: int = ...) -> BIF: ...


class FacemarkKazemi(Facemark):
    ...

class Facemark(cv2.Algorithm):
    # Functions
    def loadModel(self, model: str) -> None: ...

    @_typing.overload
    def fit(self, image: cv2.typing.MatLike, faces: cv2.typing.MatLike, landmarks: _typing.Sequence[cv2.typing.MatLike] | None = ...) -> tuple[bool, _typing.Sequence[cv2.typing.MatLike]]: ...
    @_typing.overload
    def fit(self, image: cv2.UMat, faces: cv2.UMat, landmarks: _typing.Sequence[cv2.UMat] | None = ...) -> tuple[bool, _typing.Sequence[cv2.UMat]]: ...


class FacemarkAAM(FacemarkTrain):
    ...

class FacemarkTrain(Facemark):
    ...

class FacemarkLBF(FacemarkTrain):
    ...

class BasicFaceRecognizer(FaceRecognizer):
    # Functions
    def getNumComponents(self) -> int: ...

    def setNumComponents(self, val: int) -> None: ...

    def getThreshold(self) -> float: ...

    def setThreshold(self, val: float) -> None: ...

    def getProjections(self) -> _typing.Sequence[cv2.typing.MatLike]: ...

    def getLabels(self) -> cv2.typing.MatLike: ...

    def getEigenValues(self) -> cv2.typing.MatLike: ...

    def getEigenVectors(self) -> cv2.typing.MatLike: ...

    def getMean(self) -> cv2.typing.MatLike: ...


class EigenFaceRecognizer(BasicFaceRecognizer):
    # Functions
    @classmethod
    def create(cls, num_components: int = ..., threshold: float = ...) -> EigenFaceRecognizer: ...


class FisherFaceRecognizer(BasicFaceRecognizer):
    # Functions
    @classmethod
    def create(cls, num_components: int = ..., threshold: float = ...) -> FisherFaceRecognizer: ...


class LBPHFaceRecognizer(FaceRecognizer):
    # Functions
    def getGridX(self) -> int: ...

    def setGridX(self, val: int) -> None: ...

    def getGridY(self) -> int: ...

    def setGridY(self, val: int) -> None: ...

    def getRadius(self) -> int: ...

    def setRadius(self, val: int) -> None: ...

    def getNeighbors(self) -> int: ...

    def setNeighbors(self, val: int) -> None: ...

    def getThreshold(self) -> float: ...

    def setThreshold(self, val: float) -> None: ...

    def getHistograms(self) -> _typing.Sequence[cv2.typing.MatLike]: ...

    def getLabels(self) -> cv2.typing.MatLike: ...

    @classmethod
    def create(cls, radius: int = ..., neighbors: int = ..., grid_x: int = ..., grid_y: int = ..., threshold: float = ...) -> LBPHFaceRecognizer: ...


class MACE(cv2.Algorithm):
    # Functions
    def salt(self, passphrase: str) -> None: ...

    @_typing.overload
    def train(self, images: _typing.Sequence[cv2.typing.MatLike]) -> None: ...
    @_typing.overload
    def train(self, images: _typing.Sequence[cv2.UMat]) -> None: ...

    @_typing.overload
    def same(self, query: cv2.typing.MatLike) -> bool: ...
    @_typing.overload
    def same(self, query: cv2.UMat) -> bool: ...

    @classmethod
    def load(cls, filename: str | os.PathLike[str], objname: str = ...) -> MACE: ...

    @classmethod
    def create(cls, IMGSIZE: int = ...) -> MACE: ...


class PredictCollector:
    ...

class StandardCollector(PredictCollector):
    # Functions
    def getMinLabel(self) -> int: ...

    def getMinDist(self) -> float: ...

    def getResults(self, sorted: bool = ...) -> _typing.Sequence[tuple[int, float]]: ...

    @classmethod
    def create(cls, threshold: float = ...) -> StandardCollector: ...



# Functions
def createFacemarkAAM() -> Facemark: ...

def createFacemarkKazemi() -> Facemark: ...

def createFacemarkLBF() -> Facemark: ...

@_typing.overload
def drawFacemarks(image: cv2.typing.MatLike, points: cv2.typing.MatLike, color: cv2.typing.Scalar = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def drawFacemarks(image: cv2.UMat, points: cv2.UMat, color: cv2.typing.Scalar = ...) -> cv2.UMat: ...

@_typing.overload
def getFacesHAAR(image: cv2.typing.MatLike, face_cascade_name: str, faces: cv2.typing.MatLike | None = ...) -> tuple[bool, cv2.typing.MatLike]: ...
@_typing.overload
def getFacesHAAR(image: cv2.UMat, face_cascade_name: str, faces: cv2.UMat | None = ...) -> tuple[bool, cv2.UMat]: ...

def loadDatasetList(imageList: str, annotationList: str, images: _typing.Sequence[str], annotations: _typing.Sequence[str]) -> bool: ...

@_typing.overload
def loadFacePoints(filename: str | os.PathLike[str], points: cv2.typing.MatLike | None = ..., offset: float = ...) -> tuple[bool, cv2.typing.MatLike]: ...
@_typing.overload
def loadFacePoints(filename: str | os.PathLike[str], points: cv2.UMat | None = ..., offset: float = ...) -> tuple[bool, cv2.UMat]: ...

@_typing.overload
def loadTrainingData(filename: str | os.PathLike[str], images: _typing.Sequence[str], facePoints: cv2.typing.MatLike | None = ..., delim: str = ..., offset: float = ...) -> tuple[bool, cv2.typing.MatLike]: ...
@_typing.overload
def loadTrainingData(filename: str | os.PathLike[str], images: _typing.Sequence[str], facePoints: cv2.UMat | None = ..., delim: str = ..., offset: float = ...) -> tuple[bool, cv2.UMat]: ...
@_typing.overload
def loadTrainingData(imageList: str, groundTruth: str, images: _typing.Sequence[str], facePoints: cv2.typing.MatLike | None = ..., offset: float = ...) -> tuple[bool, cv2.typing.MatLike]: ...
@_typing.overload
def loadTrainingData(imageList: str, groundTruth: str, images: _typing.Sequence[str], facePoints: cv2.UMat | None = ..., offset: float = ...) -> tuple[bool, cv2.UMat]: ...
@_typing.overload
def loadTrainingData(filename: _typing.Sequence[str], trainlandmarks: _typing.Sequence[_typing.Sequence[cv2.typing.Point2f]], trainimages: _typing.Sequence[str]) -> bool: ...





__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


# Enumerations
CALIB_USE_INTRINSIC_GUESS: int
CALIB_RECOMPUTE_EXTRINSIC: int
CALIB_CHECK_COND: int
CALIB_FIX_SKEW: int
CALIB_FIX_K1: int
CALIB_FIX_K2: int
CALIB_FIX_K3: int
CALIB_FIX_K4: int
CALIB_FIX_INTRINSIC: int
CALIB_FIX_PRINCIPAL_POINT: int
CALIB_ZERO_DISPARITY: int
CALIB_FIX_FOCAL_LENGTH: int



# Functions
@_typing.overload
def calibrate(objectPoints: _typing.Sequence[cv2.typing.MatLike], imagePoints: _typing.Sequence[cv2.typing.MatLike], image_size: cv2.typing.Size, K: cv2.typing.MatLike, D: cv2.typing.MatLike, rvecs: _typing.Sequence[cv2.typing.MatLike] | None = ..., tvecs: _typing.Sequence[cv2.typing.MatLike] | None = ..., flags: int = ..., criteria: cv2.typing.TermCriteria = ...) -> tuple[float, cv2.typing.MatLike, cv2.typing.MatLike, _typing.Sequence[cv2.typing.MatLike], _typing.Sequence[cv2.typing.MatLike]]: ...
@_typing.overload
def calibrate(objectPoints: _typing.Sequence[cv2.UMat], imagePoints: _typing.Sequence[cv2.UMat], image_size: cv2.typing.Size, K: cv2.UMat, D: cv2.UMat, rvecs: _typing.Sequence[cv2.UMat] | None = ..., tvecs: _typing.Sequence[cv2.UMat] | None = ..., flags: int = ..., criteria: cv2.typing.TermCriteria = ...) -> tuple[float, cv2.UMat, cv2.UMat, _typing.Sequence[cv2.UMat], _typing.Sequence[cv2.UMat]]: ...

@_typing.overload
def distortPoints(undistorted: cv2.typing.MatLike, K: cv2.typing.MatLike, D: cv2.typing.MatLike, distorted: cv2.typing.MatLike | None = ..., alpha: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def distortPoints(undistorted: cv2.UMat, K: cv2.UMat, D: cv2.UMat, distorted: cv2.UMat | None = ..., alpha: float = ...) -> cv2.UMat: ...
@_typing.overload
def distortPoints(undistorted: cv2.typing.MatLike, Kundistorted: cv2.typing.MatLike, K: cv2.typing.MatLike, D: cv2.typing.MatLike, distorted: cv2.typing.MatLike | None = ..., alpha: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def distortPoints(undistorted: cv2.UMat, Kundistorted: cv2.UMat, K: cv2.UMat, D: cv2.UMat, distorted: cv2.UMat | None = ..., alpha: float = ...) -> cv2.UMat: ...

@_typing.overload
def estimateNewCameraMatrixForUndistortRectify(K: cv2.typing.MatLike, D: cv2.typing.MatLike, image_size: cv2.typing.Size, R: cv2.typing.MatLike, P: cv2.typing.MatLike | None = ..., balance: float = ..., new_size: cv2.typing.Size = ..., fov_scale: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def estimateNewCameraMatrixForUndistortRectify(K: cv2.UMat, D: cv2.UMat, image_size: cv2.typing.Size, R: cv2.UMat, P: cv2.UMat | None = ..., balance: float = ..., new_size: cv2.typing.Size = ..., fov_scale: float = ...) -> cv2.UMat: ...

@_typing.overload
def initUndistortRectifyMap(K: cv2.typing.MatLike, D: cv2.typing.MatLike | None, R: cv2.typing.MatLike, P: cv2.typing.MatLike, size: cv2.typing.Size, m1type: int, map1: cv2.typing.MatLike | None = ..., map2: cv2.typing.MatLike | None = ...) -> tuple[cv2.typing.MatLike, cv2.typing.MatLike]: ...
@_typing.overload
def initUndistortRectifyMap(K: cv2.UMat, D: cv2.UMat | None, R: cv2.UMat, P: cv2.UMat, size: cv2.typing.Size, m1type: int, map1: cv2.UMat | None = ..., map2: cv2.UMat | None = ...) -> tuple[cv2.UMat, cv2.UMat]: ...

@_typing.overload
def projectPoints(objectPoints: cv2.typing.MatLike, rvec: cv2.typing.MatLike, tvec: cv2.typing.MatLike, K: cv2.typing.MatLike, D: cv2.typing.MatLike, imagePoints: cv2.typing.MatLike | None = ..., alpha: float = ..., jacobian: cv2.typing.MatLike | None = ...) -> tuple[cv2.typing.MatLike, cv2.typing.MatLike]: ...
@_typing.overload
def projectPoints(objectPoints: cv2.UMat, rvec: cv2.UMat, tvec: cv2.UMat, K: cv2.UMat, D: cv2.UMat, imagePoints: cv2.UMat | None = ..., alpha: float = ..., jacobian: cv2.UMat | None = ...) -> tuple[cv2.UMat, cv2.UMat]: ...

@_typing.overload
def solvePnP(objectPoints: cv2.typing.MatLike, imagePoints: cv2.typing.MatLike, cameraMatrix: cv2.typing.MatLike, distCoeffs: cv2.typing.MatLike, rvec: cv2.typing.MatLike | None = ..., tvec: cv2.typing.MatLike | None = ..., useExtrinsicGuess: bool = ..., flags: int = ..., criteria: cv2.typing.TermCriteria = ...) -> tuple[bool, cv2.typing.MatLike, cv2.typing.MatLike]: ...
@_typing.overload
def solvePnP(objectPoints: cv2.UMat, imagePoints: cv2.UMat, cameraMatrix: cv2.UMat, distCoeffs: cv2.UMat, rvec: cv2.UMat | None = ..., tvec: cv2.UMat | None = ..., useExtrinsicGuess: bool = ..., flags: int = ..., criteria: cv2.typing.TermCriteria = ...) -> tuple[bool, cv2.UMat, cv2.UMat]: ...

@_typing.overload
def solvePnPRansac(objectPoints: cv2.typing.MatLike, imagePoints: cv2.typing.MatLike, cameraMatrix: cv2.typing.MatLike, distCoeffs: cv2.typing.MatLike, rvec: cv2.typing.MatLike | None = ..., tvec: cv2.typing.MatLike | None = ..., useExtrinsicGuess: bool = ..., iterationsCount: int = ..., reprojectionError: float = ..., confidence: float = ..., inliers: cv2.typing.MatLike | None = ..., flags: int = ..., criteria: cv2.typing.TermCriteria = ...) -> tuple[bool, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike]: ...
@_typing.overload
def solvePnPRansac(objectPoints: cv2.UMat, imagePoints: cv2.UMat, cameraMatrix: cv2.UMat, distCoeffs: cv2.UMat, rvec: cv2.UMat | None = ..., tvec: cv2.UMat | None = ..., useExtrinsicGuess: bool = ..., iterationsCount: int = ..., reprojectionError: float = ..., confidence: float = ..., inliers: cv2.UMat | None = ..., flags: int = ..., criteria: cv2.typing.TermCriteria = ...) -> tuple[bool, cv2.UMat, cv2.UMat, cv2.UMat]: ...

@_typing.overload
def stereoCalibrate(objectPoints: _typing.Sequence[cv2.typing.MatLike], imagePoints1: _typing.Sequence[cv2.typing.MatLike], imagePoints2: _typing.Sequence[cv2.typing.MatLike], K1: cv2.typing.MatLike, D1: cv2.typing.MatLike, K2: cv2.typing.MatLike, D2: cv2.typing.MatLike, imageSize: cv2.typing.Size, R: cv2.typing.MatLike | None = ..., T: cv2.typing.MatLike | None = ..., rvecs: _typing.Sequence[cv2.typing.MatLike] | None = ..., tvecs: _typing.Sequence[cv2.typing.MatLike] | None = ..., flags: int = ..., criteria: cv2.typing.TermCriteria = ...) -> tuple[float, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike, _typing.Sequence[cv2.typing.MatLike], _typing.Sequence[cv2.typing.MatLike]]: ...
@_typing.overload
def stereoCalibrate(objectPoints: _typing.Sequence[cv2.UMat], imagePoints1: _typing.Sequence[cv2.UMat], imagePoints2: _typing.Sequence[cv2.UMat], K1: cv2.UMat, D1: cv2.UMat, K2: cv2.UMat, D2: cv2.UMat, imageSize: cv2.typing.Size, R: cv2.UMat | None = ..., T: cv2.UMat | None = ..., rvecs: _typing.Sequence[cv2.UMat] | None = ..., tvecs: _typing.Sequence[cv2.UMat] | None = ..., flags: int = ..., criteria: cv2.typing.TermCriteria = ...) -> tuple[float, cv2.UMat, cv2.UMat, cv2.UMat, cv2.UMat, cv2.UMat, cv2.UMat, _typing.Sequence[cv2.UMat], _typing.Sequence[cv2.UMat]]: ...
@_typing.overload
def stereoCalibrate(objectPoints: _typing.Sequence[cv2.typing.MatLike], imagePoints1: _typing.Sequence[cv2.typing.MatLike], imagePoints2: _typing.Sequence[cv2.typing.MatLike], K1: cv2.typing.MatLike, D1: cv2.typing.MatLike, K2: cv2.typing.MatLike, D2: cv2.typing.MatLike, imageSize: cv2.typing.Size, R: cv2.typing.MatLike | None = ..., T: cv2.typing.MatLike | None = ..., flags: int = ..., criteria: cv2.typing.TermCriteria = ...) -> tuple[float, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike]: ...
@_typing.overload
def stereoCalibrate(objectPoints: _typing.Sequence[cv2.UMat], imagePoints1: _typing.Sequence[cv2.UMat], imagePoints2: _typing.Sequence[cv2.UMat], K1: cv2.UMat, D1: cv2.UMat, K2: cv2.UMat, D2: cv2.UMat, imageSize: cv2.typing.Size, R: cv2.UMat | None = ..., T: cv2.UMat | None = ..., flags: int = ..., criteria: cv2.typing.TermCriteria = ...) -> tuple[float, cv2.UMat, cv2.UMat, cv2.UMat, cv2.UMat, cv2.UMat, cv2.UMat]: ...

@_typing.overload
def stereoRectify(K1: cv2.typing.MatLike, D1: cv2.typing.MatLike, K2: cv2.typing.MatLike, D2: cv2.typing.MatLike, imageSize: cv2.typing.Size, R: cv2.typing.MatLike, tvec: cv2.typing.MatLike, flags: int, R1: cv2.typing.MatLike | None = ..., R2: cv2.typing.MatLike | None = ..., P1: cv2.typing.MatLike | None = ..., P2: cv2.typing.MatLike | None = ..., Q: cv2.typing.MatLike | None = ..., newImageSize: cv2.typing.Size = ..., balance: float = ..., fov_scale: float = ...) -> tuple[cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike, cv2.typing.MatLike]: ...
@_typing.overload
def stereoRectify(K1: cv2.UMat, D1: cv2.UMat, K2: cv2.UMat, D2: cv2.UMat, imageSize: cv2.typing.Size, R: cv2.UMat, tvec: cv2.UMat, flags: int, R1: cv2.UMat | None = ..., R2: cv2.UMat | None = ..., P1: cv2.UMat | None = ..., P2: cv2.UMat | None = ..., Q: cv2.UMat | None = ..., newImageSize: cv2.typing.Size = ..., balance: float = ..., fov_scale: float = ...) -> tuple[cv2.UMat, cv2.UMat, cv2.UMat, cv2.UMat, cv2.UMat]: ...

@_typing.overload
def undistortImage(distorted: cv2.typing.MatLike, K: cv2.typing.MatLike, D: cv2.typing.MatLike, undistorted: cv2.typing.MatLike | None = ..., Knew: cv2.typing.MatLike | None = ..., new_size: cv2.typing.Size = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def undistortImage(distorted: cv2.UMat, K: cv2.UMat, D: cv2.UMat, undistorted: cv2.UMat | None = ..., Knew: cv2.UMat | None = ..., new_size: cv2.typing.Size = ...) -> cv2.UMat: ...

@_typing.overload
def undistortPoints(distorted: cv2.typing.MatLike, K: cv2.typing.MatLike, D: cv2.typing.MatLike, undistorted: cv2.typing.MatLike | None = ..., R: cv2.typing.MatLike | None = ..., P: cv2.typing.MatLike | None = ..., criteria: cv2.typing.TermCriteria = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def undistortPoints(distorted: cv2.UMat, K: cv2.UMat, D: cv2.UMat, undistorted: cv2.UMat | None = ..., R: cv2.UMat | None = ..., P: cv2.UMat | None = ..., criteria: cv2.typing.TermCriteria = ...) -> cv2.UMat: ...





__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


# Functions
@_typing.overload
def BIMEF(input: cv2.typing.MatLike, output: cv2.typing.MatLike | None = ..., mu: float = ..., a: float = ..., b: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def BIMEF(input: cv2.UMat, output: cv2.UMat | None = ..., mu: float = ..., a: float = ..., b: float = ...) -> cv2.UMat: ...

@_typing.overload
def BIMEF2(input: cv2.typing.MatLike, k: float, mu: float, a: float, b: float, output: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def BIMEF2(input: cv2.UMat, k: float, mu: float, a: float, b: float, output: cv2.UMat | None = ...) -> cv2.UMat: ...

def autoscaling(input: cv2.typing.MatLike, output: cv2.typing.MatLike) -> None: ...

def contrastStretching(input: cv2.typing.MatLike, output: cv2.typing.MatLike, r1: int, s1: int, r2: int, s2: int) -> None: ...

def gammaCorrection(input: cv2.typing.MatLike, output: cv2.typing.MatLike, gamma: float) -> None: ...

def logTransform(input: cv2.typing.MatLike, output: cv2.typing.MatLike) -> None: ...





#!/usr/bin/env python
'''
mouse_and_match.py [-i path | --input path: default ../data/]

Demonstrate using a mouse to interact with an image:
 Read in the images in a directory one by one
 Allow the user to select parts of an image with a mouse
 When they let go of the mouse, it correlates (using matchTemplate) that patch with the image.

 SPACE for next image
 ESC to exit
'''

# Python 2/3 compatibility
from __future__ import print_function

import numpy as np
import cv2 as cv

# built-in modules
import os
import sys
import glob
import argparse
from math import *


class App():
    drag_start = None
    sel = (0,0,0,0)

    def onmouse(self, event, x, y, flags, param):
        if event == cv.EVENT_LBUTTONDOWN:
            self.drag_start = x, y
            self.sel = (0,0,0,0)
        elif event == cv.EVENT_LBUTTONUP:
            if self.sel[2] > self.sel[0] and self.sel[3] > self.sel[1]:
                patch = self.gray[self.sel[1]:self.sel[3], self.sel[0]:self.sel[2]]
                result = cv.matchTemplate(self.gray, patch, cv.TM_CCOEFF_NORMED)
                result = np.abs(result)**3
                _val, result = cv.threshold(result, 0.01, 0, cv.THRESH_TOZERO)
                result8 = cv.normalize(result, None, 0, 255, cv.NORM_MINMAX, cv.CV_8U)
                cv.imshow("result", result8)
            self.drag_start = None
        elif self.drag_start:
            #print flags
            if flags & cv.EVENT_FLAG_LBUTTON:
                minpos = min(self.drag_start[0], x), min(self.drag_start[1], y)
                maxpos = max(self.drag_start[0], x), max(self.drag_start[1], y)
                self.sel = (minpos[0], minpos[1], maxpos[0], maxpos[1])
                img = cv.cvtColor(self.gray, cv.COLOR_GRAY2BGR)
                cv.rectangle(img, (self.sel[0], self.sel[1]), (self.sel[2], self.sel[3]), (0,255,255), 1)
                cv.imshow("gray", img)
            else:
                print("selection is complete")
                self.drag_start = None

    def run(self):
        parser = argparse.ArgumentParser(description='Demonstrate mouse interaction with images')
        parser.add_argument("-i","--input", default='../data/', help="Input directory.")
        args = parser.parse_args()
        path = args.input

        cv.namedWindow("gray",1)
        cv.setMouseCallback("gray", self.onmouse)
        '''Loop through all the images in the directory'''
        for infile in glob.glob( os.path.join(path, '*.*') ):
            ext = os.path.splitext(infile)[1][1:] #get the filename extension
            if ext == "png" or ext == "jpg" or ext == "bmp" or ext == "tiff" or ext == "pbm":
                print(infile)

                img = cv.imread(infile, cv.IMREAD_COLOR)
                if img is None:
                    continue
                self.sel = (0,0,0,0)
                self.drag_start = None
                self.gray = cv.cvtColor(img, cv.COLOR_BGR2GRAY)
                cv.imshow("gray", self.gray)
                if cv.waitKey() == 27:
                    break

        print('Done')


if __name__ == '__main__':
    print(__doc__)
    App().run()
    cv.destroyAllWindows()



__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


# Enumerations
HAAR: int
TransformTypes = int
"""One of [HAAR]"""

BM3D_STEPALL: int
BM3D_STEP1: int
BM3D_STEP2: int
Bm3dSteps = int
"""One of [BM3D_STEPALL, BM3D_STEP1, BM3D_STEP2]"""

INPAINT_SHIFTMAP: int
INPAINT_FSR_BEST: int
INPAINT_FSR_FAST: int
InpaintTypes = int
"""One of [INPAINT_SHIFTMAP, INPAINT_FSR_BEST, INPAINT_FSR_FAST]"""



# Classes
class TonemapDurand(cv2.Tonemap):
    # Functions
    def getSaturation(self) -> float: ...

    def setSaturation(self, saturation: float) -> None: ...

    def getContrast(self) -> float: ...

    def setContrast(self, contrast: float) -> None: ...

    def getSigmaSpace(self) -> float: ...

    def setSigmaSpace(self, sigma_space: float) -> None: ...

    def getSigmaColor(self) -> float: ...

    def setSigmaColor(self, sigma_color: float) -> None: ...


class WhiteBalancer(cv2.Algorithm):
    # Functions
    @_typing.overload
    def balanceWhite(self, src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def balanceWhite(self, src: cv2.UMat, dst: cv2.UMat | None = ...) -> cv2.UMat: ...


class SimpleWB(WhiteBalancer):
    # Functions
    def getInputMin(self) -> float: ...

    def setInputMin(self, val: float) -> None: ...

    def getInputMax(self) -> float: ...

    def setInputMax(self, val: float) -> None: ...

    def getOutputMin(self) -> float: ...

    def setOutputMin(self, val: float) -> None: ...

    def getOutputMax(self) -> float: ...

    def setOutputMax(self, val: float) -> None: ...

    def getP(self) -> float: ...

    def setP(self, val: float) -> None: ...


class GrayworldWB(WhiteBalancer):
    # Functions
    def getSaturationThreshold(self) -> float: ...

    def setSaturationThreshold(self, val: float) -> None: ...


class LearningBasedWB(WhiteBalancer):
    # Functions
    @_typing.overload
    def extractSimpleFeatures(self, src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def extractSimpleFeatures(self, src: cv2.UMat, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

    def getRangeMaxVal(self) -> int: ...

    def setRangeMaxVal(self, val: int) -> None: ...

    def getSaturationThreshold(self) -> float: ...

    def setSaturationThreshold(self, val: float) -> None: ...

    def getHistBinNum(self) -> int: ...

    def setHistBinNum(self, val: int) -> None: ...



# Functions
@_typing.overload
def applyChannelGains(src: cv2.typing.MatLike, gainB: float, gainG: float, gainR: float, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def applyChannelGains(src: cv2.UMat, gainB: float, gainG: float, gainR: float, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def bm3dDenoising(src: cv2.typing.MatLike, dstStep1: cv2.typing.MatLike, dstStep2: cv2.typing.MatLike | None = ..., h: float = ..., templateWindowSize: int = ..., searchWindowSize: int = ..., blockMatchingStep1: int = ..., blockMatchingStep2: int = ..., groupSize: int = ..., slidingStep: int = ..., beta: float = ..., normType: int = ..., step: int = ..., transformType: int = ...) -> tuple[cv2.typing.MatLike, cv2.typing.MatLike]: ...
@_typing.overload
def bm3dDenoising(src: cv2.UMat, dstStep1: cv2.UMat, dstStep2: cv2.UMat | None = ..., h: float = ..., templateWindowSize: int = ..., searchWindowSize: int = ..., blockMatchingStep1: int = ..., blockMatchingStep2: int = ..., groupSize: int = ..., slidingStep: int = ..., beta: float = ..., normType: int = ..., step: int = ..., transformType: int = ...) -> tuple[cv2.UMat, cv2.UMat]: ...
@_typing.overload
def bm3dDenoising(src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., h: float = ..., templateWindowSize: int = ..., searchWindowSize: int = ..., blockMatchingStep1: int = ..., blockMatchingStep2: int = ..., groupSize: int = ..., slidingStep: int = ..., beta: float = ..., normType: int = ..., step: int = ..., transformType: int = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def bm3dDenoising(src: cv2.UMat, dst: cv2.UMat | None = ..., h: float = ..., templateWindowSize: int = ..., searchWindowSize: int = ..., blockMatchingStep1: int = ..., blockMatchingStep2: int = ..., groupSize: int = ..., slidingStep: int = ..., beta: float = ..., normType: int = ..., step: int = ..., transformType: int = ...) -> cv2.UMat: ...

def createGrayworldWB() -> GrayworldWB: ...

def createLearningBasedWB(path_to_model: str = ...) -> LearningBasedWB: ...

def createSimpleWB() -> SimpleWB: ...

def createTonemapDurand(gamma: float = ..., contrast: float = ..., saturation: float = ..., sigma_color: float = ..., sigma_space: float = ...) -> TonemapDurand: ...

def dctDenoising(src: cv2.typing.MatLike, dst: cv2.typing.MatLike, sigma: float, psize: int = ...) -> None: ...

def inpaint(src: cv2.typing.MatLike, mask: cv2.typing.MatLike, dst: cv2.typing.MatLike, algorithmType: int) -> None: ...

@_typing.overload
def oilPainting(src: cv2.typing.MatLike, size: int, dynRatio: int, code: int, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def oilPainting(src: cv2.UMat, size: int, dynRatio: int, code: int, dst: cv2.UMat | None = ...) -> cv2.UMat: ...
@_typing.overload
def oilPainting(src: cv2.typing.MatLike, size: int, dynRatio: int, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def oilPainting(src: cv2.UMat, size: int, dynRatio: int, dst: cv2.UMat | None = ...) -> cv2.UMat: ...


